/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

/* Example: ex_deriv.cpp
   -------------------------------------------------------------------
   An example to demonstrate numerical differentiation
*/

#include <cmath>
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_deriv.h>
#include <o2scl/cern_deriv.h>

using namespace std;
using namespace o2scl;

class cl {

public:

  // This is the function we'll take the derivative of
  double function(double x) {
    return sin(2.0*x)+0.5;
  }
};

int main(void) {
  cl acl;
  ovector x(2);
  double xa[2];
  int i;
  void *vp=0;
  size_t tmp;
  int r1, r2, r3;
  bool done;

  test_mgr t;
  t.set_output_level(2);

  funct_mfptr_nopar<cl,void *> f1(&acl,&cl::function);

  gsl_deriv<void *> gd;
  // Note that the GSL derivative routine requires an initial stepsize
  gd.h=1.0e-3;
  cern_deriv<void *> cd;
  
  // Compute the first derivative using the gsl_deriv class and
  // verify that the answer is correct
  double d1=gd.calc(1.0,vp,f1);
  t.test_rel(d1,2.0*cos(2.0),1.0e-10,"gsl_deriv");
  
  // Compute the first derivative using the cern_deriv class and
  // verify that the answer is correct
  double d2=cd.calc(1.0,vp,f1);
  t.test_rel(d2,2.0*cos(2.0),1.0e-10,"cern_deriv");

  // Compute the second derivative also
  double d3=gd.calc2(1.0,vp,f1);
  t.test_rel(d3,-4.0*sin(2.0),5.0e-7,"gsl_deriv");
  
  double d4=cd.calc2(1.0,vp,f1);
  t.test_rel(d4,-4.0*sin(2.0),1.0e-8,"cern_deriv");

  t.report();
  return 0;
}
// End of example

