/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/collection.h>

using namespace std;
using namespace o2scl;

int o2scl::operator==(const collection::iterator &i1, 
		      const collection::iterator &i2) {
  return i1.pit==i2.pit;
}

int o2scl::operator!=(const collection::iterator &i1, 
		      const collection::iterator &i2) {
  return i1.pit!=i2.pit;
}

int o2scl::operator==(const collection::type_iterator &i1, 
		      const collection::type_iterator &i2) {
  return i1.pit==i2.pit;
}

int o2scl::operator!=(const collection::type_iterator &i1, 
		      const collection::type_iterator &i2) {
  return i1.pit!=i2.pit;
}

/*---------------------------------------------------
  io_manager functions
*/

int io_manager::is_type(io_base *iop) {
  titer ti;
  for(ti=tlist.begin();ti!=tlist.end();ti++) {
    if ((*ti)==iop) {
      return gsl_success;
    }
  }
  return gsl_enotfound;
}

int io_manager::add_type(io_base *iop) {
  
#ifdef O2SCL_NEVER_DEFINED
  // I had to remove this, because there's no way to ensure that the
  // 'correct' copy of an io_base object isn't deleted prematurely?
  
  // We ensure there aren't too many duplicates by checking the last
  // element in 'tlist' and ensuring that it doesn't have a duplicate.

  if (tlist.size()>0) {
    string comp=tlist[tlist.size()-1]->type();

    // Make sure that this isn't a virtual object
    if (comp!="io_base") {
      bool done=false;

      // Go through the list
      for(size_t i=0;done==false && i<tlist.size()-1;i++) {
	if (tlist[i]->type()==comp) {

	  // Erase the duplicate
	  titer it=tlist.begin();
	  it+=tlist.size()-1;
	  tlist.erase(it);

	  // Since we removed the type, it doesn't
	  // need to be removed again, so we change
	  // io_base::sw_store
	  tlist[i]->sw_store=1;

	  done=true;
	}
      }
    }
  }
#endif

  // Finally, add the new type
  tlist.push_back(iop);
  
  return 0;
}

int io_manager::add_type(io_base *iop, const char *t) {
  
  // Make sure it's not already there
  for(size_t i=0;i<tlist.size();i++) {
    // We return non-zero so that it's not added to the types
    // in the io_base constructor
    if (tlist[i]->type()==t) return 1;
  }

  // Finally, add the new type
  tlist.push_back(iop);
  
  return 0;
}

io_base *io_manager::get_ptr(std::string stype) {
  titer tir;
  for(tir=tlist.begin();tir!=tlist.end();tir++) {
    if (((string)(*tir)->type())==stype || 
	((string)(*tir)->type())+"[]"==stype ||
	((string)(*tir)->type())+"[][]"==stype) {
      return *tir;
    }
  }
  return 0;
}

int io_manager::remove_type(io_base *iop) {
  bool done=false;
  for (titer it=tlist.begin();done==false && it!=tlist.end();it++) {
    if (*it==iop) {
      tlist.erase(it);
      done=true;
    }
  }
  return 0;
}


/*---------------------------------------------------
  cinput functions
*/

int cinput::object_in(std::string type, in_file_format *ins, void *vp,
		      std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_void(this,ins,vp,name);
  } 
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

int cinput::object_in(std::string type, in_file_format *ins, void *vp,
		      int sz, std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_void(this,ins,vp,sz,name);
  }
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

int cinput::object_in(std::string type, in_file_format *ins, void *vp,
		      int sz, int sz2, std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_void(this,ins,vp,sz,sz2,name);
  }
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

int cinput::object_in_mem(std::string type, in_file_format *ins, void *&vp,
			  std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_mem_void(this,ins,vp,name);
  } 
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

int cinput::object_in_mem(std::string type, in_file_format *ins, void *&vp,
			  int &sz, std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_mem_void(this,ins,vp,sz,name);
  }
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

int cinput::object_in_mem(std::string type, in_file_format *ins, void *&vp,
			  int &sz, int &sz2, std::string &name) {
  if (cop->iom->get_ptr(type)!=0) {
    return cop->iom->get_ptr(type)->object_in_mem_void(this,ins,vp,sz,sz2,
						       name);
  }
  O2SCL_ERR_RET("Failed to find type.",gsl_einval);
}

/*---------------------------------------------------
  coutput functions
*/

int coutput::object_out(std::string type, out_file_format *outs,
			void *op, int sz, int sz2, std::string name) {
  io_base *iob=cop->iom->get_ptr(type);
  if (iob==0) {
    string s="Failed to get IO pointer for type '"+type+
      "' in coutput::object_out().";
    O2SCL_ERR_RET(s.c_str(),gsl_efailed);
  }
  return iob->object_out_void(this,outs,op,sz,sz2,name);
}

/*---------------------------------------------------
  collection functions
*/

void collection::clear() {
  piter pit;
  // Proceed through the collection and remove the 
  // data for each pointer and then disown the pointer.
  // Let the data in plist delete itself.
  for(pit=plist.begin();pit!=plist.end();pit++) {
    if (pit->second.owner==true) {
      if (iom->is_type(pit->second.iop)!=gsl_success) {
	O2SCL_ERR2("Trying to remove with a non-existent type ",
		   "in collection::clear().",gsl_efailed);
      } else {
	if (pit->second.size==0 && pit->second.size2==0) {
	  pit->second.iop->remove(pit->second.data);
	} else if (pit->second.size2==0) {
	  pit->second.iop->remove_arr(pit->second.data);
	} else {
	  pit->second.iop->remove_2darr(pit->second.data,pit->second.size);
	}
      }
      pit->second.owner=false;
      pit=plist.begin();
    }
  }
  plist.clear();
  return;
}

int collection::out_one(std::string fname, string stype, string name,
			void *vp, int sz, int sz2) {
  ofstream fot;
  fot.setf(ios::scientific);
  text_out_file *tof=new text_out_file(fname,&fot);
  int ret=out_one(tof,stype,name,vp,sz,sz2);
  delete tof;
  return ret;
}

int collection::out_one(out_file_format *outs, string stype, string name,
			void *vp, int sz, int sz2) {
  io_base *iop=iom->get_ptr(stype);
  coutput co(this);
  
  if (iop!=0) {
    outs->init_file();
    outs->start_object(iop->type(),name);
    iop->out_wrapper(&co,outs,vp,sz,sz2);
    outs->end_object();
    outs->end_line();
    
  } else {
    O2SCL_ERR("Type not found in out_one().",gsl_einval);
    return gsl_einval;
  }
  return gsl_success;
}

int collection::summary(ostream *outs, bool show_addresses) {
  piter pit;
  string *typs;
  int i, nout;

  if (plist.size()>0) {
    (*outs) << "Parameters:" << endl;
    if (show_addresses) {
      (*outs) << "type name size size2 owner data iop" << endl;
    } else {
      (*outs) << "type name size size2 owner" << endl;
    }
    typs=new string[plist.size()];
    i=0;
    for(pit=plist.begin();pit!=plist.end();pit++) {
      typs[i]=((string)pit->second.iop->type())+" "+
	pit->first+" "+o2scl::itos(pit->second.size)+" "+
	o2scl::itos(pit->second.size2)+" "+
	o2scl::itos(pit->second.owner);
      if (show_addresses) {
	typs[i]+=((string)" ")+o2scl::ptos(pit->second.data);
	typs[i]+=((string)" ")+o2scl::ptos(pit->second.iop);
      }
      i++;
    }
    vector<string> stypes;
    o2scl::screenify(plist.size(),typs,stypes);
    nout=stypes.size();
    for(i=0;i<nout;i++) {
      (*outs) << stypes[i] << endl;
    }
    delete[] typs;
  } else {
    (*outs) << "No parameters." << endl;
  }
  
  return gsl_success;
}

int collection::fout(std::string filename) {
  text_out_file *tof=new text_out_file(filename);
  int ret=fout(tof);
  delete tof;
  return ret;
}

int collection::get_type(text_out_file &tof, std::string stype, 
			 std::string name) {
  
  coutput co(this);
  piter pit=plist.find(name);
  if (pit!=plist.end()) {
    
    io_base *iop=pit->second.iop;
    if (iop!=0 && stype==iop->type()) {
      iop->out_wrapper(&co,&tof,pit->second.data,
		       pit->second.size,pit->second.size2);
      tof.end_line();
    }
    
    return gsl_success;
  }
  return gsl_enotfound;
  
}

int collection::get(text_out_file &tof, std::string &stype, 
		    std::string name) {
  coutput co(this);
  piter pit=plist.find(name);
  if (pit!=plist.end()) {
    
    io_base *iop=pit->second.iop;
    if (iop!=0) {
      stype=iop->type();
      iop->out_wrapper(&co,&tof,pit->second.data,
		       pit->second.size,pit->second.size2);
      tof.end_line();
    }
    
    return gsl_success;
  }
  return gsl_enotfound;
}

int collection::set(std::string name, std::string s, bool err_on_notfound) {
  istringstream iss(s);
  text_in_file tif(&iss);
  return set(name,tif,err_on_notfound);
}

int collection::set(std::string name, text_in_file &tif, 
		    bool err_on_notfound) {
  int sz, sz2;
  piter pit=plist.find(name);
  int ret;
  cinput ci(this);
  if (pit!=plist.end()) {
    io_base *iop=pit->second.iop;
    //if (iop!=0 && iop->type()==stype) {
    if (iop!=0) {
      void *vp=pit->second.data;
      //      if (stype==iop->type()) {
      ret=iop->in_wrapper(&ci,&tif,vp);
      //      } else if (stype==((string)iop->type()+"[]")) {
      //	ret=iop->in_wrapper(&ci,&tif,vp,sz);
      //      } else {
      //	ret=iop->in_wrapper(&ci,&tif,vp,sz,sz2);
      //      }
      return gsl_success;
    } else {
      O2SCL_ERR_RET("I/O pointer is 0 in collection::set().",gsl_efailed);
    }
  }
  if (err_on_notfound) {
    O2SCL_ERR_RET("Could't find parameter in collection::set().",
		  gsl_enotfound);
  }
  return gsl_enotfound;
}

int collection::fout(out_file_format *outs) {
  piter pit;

  outs->init_file();
  
  coutput co(this);
  
  // Add all of the items in the collection
  // to the ptr_map, so that pointers to items
  // in the list are found by pointer_out(). Set
  // "bool output" to true, since we're going to
  // output them ahead of time.
  for(pit=plist.begin();pit!=plist.end();pit++) {
    pointer_output po={(*pit).first,&(*pit).second,true};
    co.ptr_map.insert(make_pair((*pit).second.data,po));
  }

  // Output all of the static data
  io_type_info iti;
  iti.static_fout(&co,outs);
  
  // First, output all of the list items so they're
  // in alphabetical order. 
  for(pit=plist.begin();pit!=plist.end();pit++) {
    if (pit->second.size==0 && pit->second.size2==0) {
      outs->start_object(pit->second.iop->type(),pit->first);
    } else if (pit->second.size2==0) {
      outs->start_object
	(((string)(pit->second.iop->type()))+"[]",pit->first);
    } else {
      outs->start_object
	(((string)(pit->second.iop->type()))+"[][]",pit->first);
    }
    pit->second.iop->out_wrapper(&co,outs,pit->second.data,
				 pit->second.size,pit->second.size2);
    outs->end_object();
    outs->end_line();
  }

  // Now go through ptr_map and output the remaining
  // objects.
  co.pointer_map_fout(outs);

  // Remove all "ptr" entries from collection
  for(pit=plist.begin();pit!=plist.end();pit++) {
    string tmp=pit->first;
    if (tmp[0]=='p' && tmp[1]=='t' && tmp[2]=='r') {
      plist.erase(pit);
      pit=plist.begin();
    }
  }
 
  return gsl_success;
}

int collection::fin(std::string file_name, bool overwrt, int verbose) {
  file_detect fd;
  in_file_format *iff=fd.open(file_name.c_str());
  if (!iff) {
    O2SCL_ERR_RET("Failed to open file in collection::fin().",
		  gsl_efailed);
  }
  int ret=fin(iff,overwrt,verbose);
  fd.close();
  return ret;
}

int collection::fin(in_file_format *ins, bool overwrt, int verbose) {
  string stemp, sname, line, stype, v;
  int sz, sz2, w, i, test;
  void *vptemp;
  io_base *iop;
  cinput ci(this);

  if (verbose>0) cout << "verbose>0 in fin()." << endl;

  ins->init_file();
  
  test=0;
  // Fill sname with a string so it knows that we're expecting a name
  sname="name";

  while (test==0 && ins->start_object(stype,sname)==0) {
    if (stype=="static") { 
      stype=sname;
      ins->word_in(sname);
      
      if (verbose>0) {
	cout << "Reading static type: " << stype << ", name: " 
	     << sname << " " << flush;
      }
      iop=iom->get_ptr(stype);
      if (iop==0) {
	ins->skip_object();
	if (verbose>0) cout << "skipped." << endl;
      } else {
	iop->stat_in_noobj(&ci,ins);
	if (verbose>0) cout << "read." << endl;
	ins->end_object();
      }
      
    } else {
      
      if (verbose>0) {
	cout << "Reading type: " << stype << ", name: " 
	     << sname << " " << flush;
      }
      iop=iom->get_ptr(stype);
      if (iop==0) {
	ins->skip_object();
	if (verbose>0) cout << "skipped." << endl;
      } else {
	vptemp=0;
	if (stype==iop->type()) {
	  test=iop->in_wrapper(&ci,ins,vptemp);
	  sz=0;
	  sz2=0;
	  if (verbose>0) cout << "scalar." << endl;

	} else if (stype==((string)iop->type()+"[]")) {
	  test=iop->in_wrapper(&ci,ins,vptemp,sz);
	  sz2=0;
	  if (verbose>0) cout << "array." << endl;
	} else {
	  test=iop->in_wrapper(&ci,ins,vptemp,sz,sz2);
	  if (verbose>0) cout << "matrix." << endl;
	}
	if (verbose>0) cout << "End object." << endl;
	ins->end_object();
	if (test==0) {
	  add_void(sname,iop,vptemp,sz,sz2,overwrt,1);
	} else {
	  if (verbose>0) {
	    cout << "Failed to add in fin()." << endl;
	  }
	}
      }
    }

    // Fill sname with a string so it knows that we're expecting a name
    sname="name";
    
  }

  if (verbose>0) {
    cout << "Calling assign pointers in fin()." << endl;
  }
  ci.assign_pointers(this);

  if (test!=0) {
    O2SCL_ERR("Input failed in collection::fin().",5);
    return 5;
  }
  
  if (verbose>0) {
    cout << "Done with fin(). " << plist.size() << endl;
  }
  return gsl_success;
}

int coutput::pointer_lookup(void *vp, string &name, 
			    collection_entry *& ep) {
  collection::piter pit;
  
  if (cop->plist.size()==0) {
    return gsl_enotfound;
  }
  
  for(pit=cop->plist.begin();pit!=cop->plist.end();pit++) {
    if (pit->second.data==vp) {
      name=pit->first;
      ep=&pit->second;
      return 0;
    }
  }
  return gsl_enotfound;
}

int collection::test_type(o2scl::test_mgr &t, string stype, void *obj,
                          void *&newobj, bool scrout) {
  /*
  bool done;
  string type2, name2, sversion, st;
  int ret;
 
  io_type_info iti;
  t.test_gen(iti.is_type(stype)==gsl_success,"Existence of type.");
  if (iti.is_type(stype)!=gsl_success) {
    return -2;
  }
 
  ostringstream *os=new ostringstream;
  os->setf(ios::scientific);
  
  text_out_file *tof=new text_out_file(os);
  ret=out_one(tof,stype,"test",obj);
  t.test_gen(ret==0,"out_one() success");
  delete tof;

  string stmp=os->str();
  const char *ctmp=stmp.c_str();
  delete os;
  if (scrout) cout << ctmp;

  istringstream *is=new istringstream(ctmp);
  text_in_file *tif=new text_in_file(is);

  int sz, sz2;
  ret=in_one(tif,stype,name2,newobj,sz,sz2);
  t.test_gen(ret==0,"in_one() success");
  t.test_gen(sz==0,"input size");
  t.test_gen(sz2==0,"input size2");
  delete tif;
  delete is;
  
  t.test_str(name2,"test","Name");
  */
  
  return gsl_success;
}

int coutput::pointer_map_fout(out_file_format *out) {
  bool done;
  pmiter pit;
  collection_entry *ep;
  io_base *iop;

  if (ptr_map.size()!=0) {
    done=false;
    while(done==false) {
      done=true;
      for(pit=ptr_map.begin();pit!=ptr_map.end();pit++) {
	if (pit->second.output==false) {
	  done=false;
	  
	  ep=pit->second.ep;
	  iop=ep->iop;
	  
	  out->start_object(iop->type(),pit->second.name);
	  ep->iop->out_wrapper(this,out,pit->first,ep->size,ep->size2);
	  out->end_object();
	  pit->second.output=true;
	  
	  pit=ptr_map.begin();
	  done=true;
	}
      }
    }
  }

  return 0;
}
  
int collection::add_void(string name, io_base *tio, void *data, int sz, int sz2,
		    bool overwrt, bool owner) {
  // We can't put this here because this function is called by
  // internal functions which output pointers
  //  if (name[0]=='p' && name[1]=='t' && name[2]=='r') {
  //    return gsl_einval;
  //  }

  piter pit;
  if (overwrt==true) {
    pit=plist.find(name);
    if (pit!=plist.end()) {
      (*pit).second.iop=tio;
      (*pit).second.data=data;
      (*pit).second.size=sz;
      (*pit).second.size2=sz2;
      (*pit).second.owner=owner;
      return gsl_success;
    }
  }
  collection_entry p;
  p.data=data;
  p.size=sz;
  p.size2=sz2;
  p.owner=owner;
  p.iop=tio;

  plist.insert(make_pair(name,p));
  return gsl_success;
}

int collection::add_void(string name, std::string stype, 
		    void *data, int sz, int sz2,
		    bool overwrt, bool owner) {

  io_base *tio=iom->get_ptr(stype);
  return add_void(name,tio,data,sz,sz2,overwrt,owner);
}

int collection::get_void(string tname, void *&vec) {
  piter pit=plist.find(tname);
  if (pit!=plist.end()) {
    vec=pit->second.data;
    return gsl_success;
  }
  // 1/9/05 We choose not to set an error here in case
  // the user is legitimately looking for something 
  // that they don't expect to find.
  //  O2SCL_ERR(tname+" not found in get().",not_found);
  return gsl_enotfound;
}

int collection::disown(string tname) {
  piter pit=plist.find(tname);
  if (pit!=plist.end()) {
    pit->second.owner=false;
    return gsl_success;
  }
  return gsl_enotfound;
}

int collection::remove(string tname) {
  piter pit;
  
  pit=plist.find(tname);
  if (pit!=plist.end()) {
    if (pit->second.owner==true) {
      if (iom->is_type(pit->second.iop)!=gsl_success) {
	O2SCL_ERR("Trying to remove with a non-existent type.",gsl_efailed);
      }
      if (pit->second.size==0 && pit->second.size2==0) {
	pit->second.iop->remove(pit->second.data);
      } else if (pit->second.size2==0) {
	pit->second.iop->remove_arr(pit->second.data);
      } else {
	pit->second.iop->remove_2darr(pit->second.data,pit->second.size);
      }
    }
    plist.erase(pit);
    return gsl_success;
  }
  return gsl_enotfound;
}

int collection::size() {
  return plist.size();
}

collection::~collection() {

  piter pit;
  // Proceed through the collection and remove the 
  // data for each pointer and then disown the pointer.
  // Delete the list at the end.
  for(pit=plist.begin();pit!=plist.end();pit++) {
    if (pit->second.owner==true) {
      if (iom->is_type(pit->second.iop)!=gsl_success) {
	O2SCL_ERR2("Trying to remove with a non-existent type ",
		   "in collection::~collection().",gsl_efailed);
      } else {
	if (pit->second.size==0 && pit->second.size2==0) {
	  pit->second.iop->remove(pit->second.data);
	} else if (pit->second.size2==0) {
	  pit->second.iop->remove_arr(pit->second.data);
	} else {
	  pit->second.iop->remove_2darr(pit->second.data,pit->second.size);
	}
      }
      pit->second.owner=false;
      pit=plist.begin();
    }
  }

  plist.clear();
}

int cinput::assign_pointers(collection *co) {
  void *vp=0;
  
  ipiter ipt;
  
  if (input_ptrs.begin()!=input_ptrs.end()) {
    for(ipt=input_ptrs.begin();ipt!=input_ptrs.end();ipt++) {

      {
	collection::piter pit=co->plist.find(ipt->name);
	if (pit!=co->plist.end()) {
	  vp=pit->second.data;
	}
      }
      
      *(ipt->ptr)=vp;
    }
  }

  return 0;
}

/*---------------------------------------------------
  io_type_info functions
*/

io_type_info::io_type_info() : io_base(1) {
}

io_type_info::~io_type_info() {
}

int io_type_info::is_type(string stype) {
  titer ti;
  for(ti=iom->tlist.begin();ti!=iom->tlist.end();ti++) {
    if ((*ti)->type()==stype) {
      return gsl_success;
    }
  }
  O2SCL_ERR("Type not found in is_type().",gsl_enotfound);
  return gsl_enotfound;
}

int io_type_info::clear_types() {
  iom->tlist.clear();
  return gsl_success;
}

int io_type_info::remove_type(string stype) {
  titer ti;
  bool found=false;
  io_base *iop=iom->get_ptr(stype);
  if (iop==0) return gsl_enotfound;
  if (iom->tlist.size()==0) return gsl_success;
  for(ti=iom->tlist.begin();ti!=iom->tlist.end();ti++) {
    if ((*ti)==iop) {
      iom->tlist.erase(ti);
      found=true;
      if (iom->tlist.begin()==iom->tlist.end()) return gsl_success;
      ti=iom->tlist.begin();
    }
  }
  if (found) return gsl_success;
  return gsl_enotfound;
}

void io_type_info::type_summary(ostream *outs, bool pointers) {
  titer ti;
  if (iom->tlist.begin()==iom->tlist.end()) {
    (*outs) << "No types." << endl;
    return;
  }
  (*outs) << iom->tlist.size() << " types." << endl;
  string *typs=new string[iom->tlist.size()];
  int i=0, nout;
  for(ti=iom->tlist.begin();ti!=iom->tlist.end();ti++) {
    typs[i]=o2scl::itos(i+1)+". "+(*ti)->type()+" ";
    if (pointers==true) typs[i]+=" "+o2scl::ptos(*ti);
    i++;
  }
  vector<string> stypes;
  o2scl::screenify(iom->tlist.size(),typs,stypes);
  nout=stypes.size();
  (*outs) << "Registered types: " << endl;
  for(i=0;i<nout;i++) {
    (*outs) << stypes[i] << endl;
  }
  delete[] typs;
}

int io_type_info::static_fout(coutput *co, out_file_format *outf) {
  titer it;
  
  for(it=iom->tlist.begin();it!=iom->tlist.end();it++) {
    if ((*it)->has_static_data()) {
      outf->word_out("static");
      outf->word_out((*it)->type());
      (*it)->stat_out_noobj(co,outf);
    }
  }
  return 0;
}

int io_type_info::static_fout_restricted(coutput *co, out_file_format *outf,
					 set<string,string_comp> list) {
  titer it;
  
  for(it=iom->tlist.begin();it!=iom->tlist.end();it++) {
    if ((*it)->has_static_data() && list.find((*it)->type())==list.end()) {
      outf->word_out("static");
      outf->word_out((*it)->type());
      (*it)->stat_out_noobj(co,outf);
    }
  }
  return 0;
}

/*---------------------------------------------------
  io_base functions
*/

io_manager *io_base::iom=0;
int io_base::objs_count=0;

io_base::io_base(int sw) {
  if (iom==0) {
    iom=new io_manager;
  }
  if (sw==0) {
    iom->add_type(this);
    sw_store=0;
  } else {
    sw_store=1;
  }
  objs_count++;
}

io_base::io_base(const char *t) {
  if (iom==0) {
    iom=new io_manager;
  }
  sw_store=iom->add_type(this,t);
  objs_count++;
}

io_base::~io_base() {
  if (sw_store==0) {
    iom->remove_type(this);
  }
  if (objs_count==1 && iom!=0) {
    delete iom;
    iom=0;
  }
  objs_count--;
}

int io_base::pointer_out(coutput *co, out_file_format *outs, void *ptr, 
			 std::string stype) {
  string name;
  collection_entry *ep=0;
  
  if (ptr==0) {
    outs->word_out("null");
    outs->word_out("null");
    return 0;
  }

  //  cout << "\nOutputing non-zero pointer: " << stype << endl;
  
  coutput::pmiter pit;
  pit=co->ptr_map.find(ptr);
  
  // It's already in the ptr_map, output the type and name
  if (pit!=co->ptr_map.end()) {

    outs->word_out(stype);
    outs->word_out((*pit).second.name);

  } else {

    // Otherwise, add to ptr_map with the appropriate values
    // and output the new type and name
    
    if (co->pointer_lookup(ptr,name,ep)==0) {
      
      // If the object being pointed to is
      // already in the collection, then add to
      // the ptr_map with the "output=true"
      
      // Changed this to true, 9/26/04. Is this right?
      pointer_output po={name,ep,true};
      co->ptr_map.insert(make_pair(ptr,po));
      outs->word_out(stype);
      outs->word_out(po.name);
      co->npointers++;
    } else {

      // Otherwise, add it to the collection before
      // adding to ptr_map

      io_base *ioptr=iom->get_ptr(stype);
      co->cop->add_void(((string)"ptr")+o2scl::itos(co->npointers),
		   ioptr,ptr,0,0);
      co->pointer_lookup(ptr,name,ep);

      // Changed this to true, 9/26/04. Is this right?
      pointer_output po={name,ep,true};
      co->ptr_map.insert(make_pair(ptr,po));
      outs->word_out(stype);
      outs->word_out(po.name);
      co->npointers++;      
    }
  }

  return 0;
}

int io_base::pointer_in(cinput *ci, in_file_format *ins, void **pp, 
			string &stype) {
  string name;
  ins->word_in(stype);
  ins->word_in(name);
  cout << "Pointer in: " << stype << " " << name << endl;
  if (stype!="null") {
    //    *pp=0;
    //    stype="";
    //  } else {
    pointer_input pi={name,pp,stype};
    ci->input_ptrs.push_back(pi);
  }
  return 0;
}

int io_base::stat_in_noobj(cinput *co, in_file_format *ins) {
  return 0;
}
  
int io_base::stat_out_noobj(coutput *co, out_file_format *outs) {
  return 0;
}
  
int io_base::in_wrapper(cinput *co, in_file_format *ins, void *&vp) 
{ return 0; }
  
int io_base::in_wrapper(cinput *co, in_file_format *ins, void *&vp, 
			int &sz) { return 0; }

int io_base::in_wrapper(cinput *co, in_file_format *ins, void *&vp, 
			int &sz, int &sz2) { return 0; }

int io_base::out_wrapper(coutput *co, out_file_format *outs, 
			 void *vp, int sz, int sz2) {
  return 0;
}

int io_base::object_in_void(cinput *cin, in_file_format *ins, void *op,
			    std::string &name) {
  return 0;
}
    
int io_base::object_in_void(cinput *cin, in_file_format *ins, void *op, 
			    int sz, std::string &name) {
  return 0;
}
    
int io_base::object_in_void(cinput *cin, in_file_format *ins, void *op, 
			    int sz, int sz2, std::string &name) {
  return 0;
}
    
int io_base::object_in_mem_void(cinput *cin, in_file_format *ins, void *&op,
				std::string &name) {
  return 0;
}
    
int io_base::object_in_mem_void(cinput *cin, in_file_format *ins, void *&op, 
				int &sz, std::string &name) {
  return 0;
}

int io_base::object_in_mem_void(cinput *cin, in_file_format *ins, void *&op, 
				int &sz, int &sz2, std::string &name) {
  return 0;
}
    
int io_base::object_out_void(coutput *cout, out_file_format *outs, 
			     void *op, int sz, int sz2, std::string name) {
  return 0;
}

/*---------------------------------------------------
  bool_io functions
*/

template<> int io_tlate<bool>::input(cinput *co, in_file_format *ins, 
				     bool *dp) {
  ins->bool_in(*dp);
  return 0;
}

template<> int io_tlate<bool>::output(coutput *co, out_file_format *outs, 
				      bool *dp) {
  outs->bool_out(*dp);
  return 0;
}

template<> const char *io_tlate<bool>::type() { return "bool"; }

/*---------------------------------------------------
  char_io functions
*/

template<> int io_tlate<char>::input(cinput *co, in_file_format *ins, 
				     char *dp) {
  ins->char_in(*dp);
  return 0;
}

template<> int io_tlate<char>::output(coutput *co, out_file_format *outs, 
				      char *dp) {
  outs->char_out(*dp);
  return 0;
}

template<> const char *io_tlate<char>::type() { return "char"; }

/*---------------------------------------------------
  double_io functions
*/

template<> int io_tlate<double>::input(cinput *co, in_file_format *ins, 
				       double *dp) {
  ins->double_in(*dp);
  return 0;
}

template<> int io_tlate<double>::output(coutput *co, out_file_format *outs, 
					double *dp) {
  outs->double_out(*dp);
  return 0;
}

template<> const char *io_tlate<double>::type() { return "double"; }

/*---------------------------------------------------
  int_io functions
*/

template<> int io_tlate<int>::input(cinput *co, in_file_format *ins, 
				    int *dp) {
  ins->int_in(*dp);
  return 0;
}

template<> int io_tlate<int>::output(coutput *co, out_file_format *outs, 
				     int *dp) {
  outs->int_out(*dp);
  return 0;
}

template<> const char *io_tlate<int>::type() { return "int"; }

/*---------------------------------------------------
  long_io functions
*/

template<> int io_tlate<unsigned long int>::input
(cinput *co, in_file_format *ins, unsigned long int *dp) {
  ins->long_in(*dp);
  return 0;
}

template<> int io_tlate<unsigned long int>::output
(coutput *co, out_file_format *outs, unsigned long int *dp) {
  outs->long_out(*dp);
  return 0;
}

template<> const char *io_tlate<unsigned long int>::type() { return "long"; }

/*---------------------------------------------------
  string_io functions
*/

template<> int io_tlate<string>::input(cinput *co, 
				       in_file_format *ins, 
				       std::string *dp) {
  ins->string_in(*dp);
  return 0;
}

template<> int io_tlate<string>::output(coutput *co, out_file_format *outs, 
					std::string *dp) {
  outs->string_out(*dp);
  return 0;
}

template<> const char *io_tlate<std::string>::type() { return "string"; }

/*---------------------------------------------------
  word_io functions
*/

int word_io_type::input(cinput *co, in_file_format *ins, 
			std::string *dp) {
  ins->word_in(*dp);
  return 0;
}

int word_io_type::output(coutput *co, out_file_format *outs, 
			 std::string *dp) {
  outs->word_out(*dp);
  return 0;
}

#ifdef O2SCL_NEVER_DEFINED

/*---------------------------------------------------
  ovector_io functions
*/

template<> int io_tlate<ovector>::input(cinput *co, in_file_format *ins, 
					ovector *dp) {
  int sz;
  ins->int_in(sz);
  dp->allocate(sz);
  for(size_t i=0;i<((size_t)sz);i++) {
    ins->double_in((*dp)[i]);
  }
  return 0;
}

template<> int io_tlate<ovector>::output(coutput *co, out_file_format *outs, 
					 ovector *dp) {
  outs->int_out(dp->size());
  for(size_t i=0;i<dp->size();i++) {
    outs->double_out((*dp)[i]);
  }
  return 0;
}

template<> const char *io_tlate<ovector>::type() { return "ovector"; }

/*---------------------------------------------------
  omatrix_io functions
*/

template<> int io_tlate<omatrix>::input(cinput *co, in_file_format *ins, 
					omatrix *dp) {
  int sz, sz2;
  ins->int_in(sz);
  ins->int_in(sz2);
  dp->allocate(sz,sz2);
  for(size_t i=0;i<((size_t)sz);i++) {
    for(size_t j=0;j<((size_t)sz2);j++) {
      ins->double_in((*dp)[i][j]);
    }
  }
  return 0;
}

template<> int io_tlate<omatrix>::output(coutput *co, out_file_format *outs, 
					 omatrix *dp) {
  outs->int_out(dp->rows());
  outs->int_out(dp->cols());
  for(size_t i=0;i<dp->rows();i++) {
    for(size_t j=0;j<dp->cols();j++) {
      outs->double_out((*dp)[i][j]);
    }
  }
  return 0;
}

template<> const char *io_tlate<omatrix>::type() { return "omatrix"; }

/*---------------------------------------------------
  ovector_int_io functions
*/

template<> int io_tlate<ovector_int>::input(cinput *co, in_file_format *ins, 
					    ovector_int *dp) {
  int sz;
  ins->int_in(sz);
  dp->allocate(sz);
  for(size_t i=0;i<((size_t)sz);i++) {
    ins->int_in((*dp)[i]);
  }
  return 0;
}

template<> int io_tlate<ovector_int>::output(coutput *co, 
					     out_file_format *outs, 
					     ovector_int *dp) {
  outs->int_out(dp->size());
  for(size_t i=0;i<dp->size();i++) {
    outs->int_out((*dp)[i]);
  }
  return 0;
}

template<> const char *io_tlate<ovector_int>::type() 
{ return "ovector_int"; }

/*---------------------------------------------------
  omatrix_int_io functions
*/

template<> int io_tlate<omatrix_int>::input(cinput *co, in_file_format *ins, 
					    omatrix_int *dp) {
  int sz, sz2;
  ins->int_in(sz);
  ins->int_in(sz2);
  dp->allocate(sz,sz2);
  for(size_t i=0;i<((size_t)sz);i++) {
    for(size_t j=0;j<((size_t)sz2);j++) {
      //      ins->int_in((*dp)[i][j]);
    }
  }
  return 0;
}

template<> int io_tlate<omatrix_int>::output(coutput *co, 
					     out_file_format *outs, 
					     omatrix_int *dp) {
  outs->int_out(dp->rows());
  outs->int_out(dp->cols());
  for(size_t i=0;i<dp->rows();i++) {
    for(size_t j=0;j<dp->cols();j++) {
      //      outs->int_out((*dp)[i][j]);
    }
  }
  return 0;
}

template<> const char *io_tlate<omatrix_int>::type() 
{ return "omatrix_int"; }

/*---------------------------------------------------
  ovector_cx_io functions
*/

template<> int io_tlate<ovector_cx>::input(cinput *co, in_file_format *ins, 
					   ovector_cx *dp) {
  int sz;
  ins->int_in(sz);
  dp->allocate(sz);
  for(size_t i=0;i<((size_t)sz);i++) {
    double re,im;
    ins->double_in(re);
    ins->double_in(im);
    gsl_complex gc={{re,im}};
    dp->set(i,gc);
  }
  return 0;
}

template<> int io_tlate<ovector_cx>::output(coutput *co, 
					    out_file_format *outs, 
					    ovector_cx *dp) {
  outs->int_out(dp->size());
  for(size_t i=0;i<dp->size();i++) {
    outs->double_out(dp->real(i));
    outs->double_out(dp->imag(i));
  }
  return 0;
}

template<> const char *io_tlate<ovector_cx>::type() { return "ovector_cx"; }

/*---------------------------------------------------
  omatrix_cx_io functions
*/

template<> int io_tlate<omatrix_cx>::input(cinput *co, in_file_format *ins, 
					   omatrix_cx *dp) {
  int sz, sz2;
  ins->int_in(sz);
  ins->int_in(sz2);
  dp->allocate(sz,sz2);
  for(size_t i=0;i<((size_t)sz);i++) {
    for(size_t j=0;j<((size_t)sz2);j++) {
      double re, im;
      ins->double_in(re);
      ins->double_in(im);
      gsl_complex gc={{re,im}};
      dp->set(i,j,gc);
    }
  }
  return 0;
}

template<> int io_tlate<omatrix_cx>::output(coutput *co, 
					    out_file_format *outs, 
					    omatrix_cx *dp) {
  outs->int_out(dp->rows());
  outs->int_out(dp->cols());
  for(size_t i=0;i<dp->rows();i++) {
    for(size_t j=0;j<dp->cols();j++) {
      outs->double_out(dp->real(i,j));
      outs->double_out(dp->imag(i,j));
    }
  }
  return 0;
}

template<> const char *io_tlate<omatrix_cx>::type() { return "omatrix_cx"; }

#endif

