/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/table3d.h>

using namespace std;
using namespace o2scl;

table3d::table3d() {
  xy_set=false;
  size_set=false;
  has_slice=false;
  bimp1=&dim1;
  bimp2=&dim2;
}

table3d::~table3d() {
  tree.clear();
  for(size_t i=0;i<list.size();i++) {
    list[i].free();
  }
  list.clear();
  
  if (xy_set) {
    xval.free();
    yval.free();
  }
}

int table3d::set_size(size_t nx, size_t ny) {
  if ((has_slice && size_set) || xy_set) {
    O2SCL_ERR_RET("Size cannot be reset in table3d::set_xy().",
		o2scl::gsl_einval);
  }
  if (xy_set) {
    xval.free();
    yval.free();
  }
  numx=nx;
  numy=ny;
  size_set=true;
  return 0;
}

int table3d::set(size_t ix, size_t iy, std::string name, double val) {
  size_t z=0;
  int ret=lookup_slice(name,z);
  if (ret==0) {
    (list[z])[ix][iy]=val;
    return o2scl::gsl_success;
  } 
  O2SCL_ERR_RET("Failed to find slice in set().",o2scl::gsl_enotfound);
}

int table3d::set_val(double x, double y, std::string name, double val) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    (list[z])[ix][iy]=val;
    return o2scl::gsl_success;
  } 
  O2SCL_ERR("Failed to find slice in set().",o2scl::gsl_einval);
  return o2scl::gsl_einval;
}
    
int table3d::set_val_ret(double &x, double &y, std::string name, double val) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];

  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    (list[z])[ix][iy]=val;
    return o2scl::gsl_success;
  } 
  O2SCL_ERR("Failed to find slice in set().",o2scl::gsl_einval);
  return o2scl::gsl_einval;
}
    
int table3d::set(size_t ix, size_t iy, size_t z, double val) {
  (list[z])[ix][iy]=val;
  return o2scl::gsl_success;
}

int table3d::set_val(double x, double y, size_t z, double val) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];

  (list[z])[ix][iy]=val;
  return o2scl::gsl_success;
}
    
int table3d::set_val_ret(double &x, double &y, size_t z, double val) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);

  (list[z])[ix][iy]=val;
  return o2scl::gsl_success;
}
    
double &table3d::get(size_t ix, size_t iy, std::string name) {
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}

const double &table3d::get(size_t ix, size_t iy, std::string name) const {
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}

double &table3d::get_val_ret(double &x, double &y, std::string name) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}

const double &table3d::get_val_ret(double &x, double &y, 
				   std::string name) const {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}

double &table3d::get_val(double x, double y, std::string name) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}
    
const double &table3d::get_val(double x, double y, std::string name) const {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret==0) {
    return (list[z])[ix][iy];
  } 
  O2SCL_ERR("Failed to find slice in get().",o2scl::gsl_einval);
  return list[0][0][0];
}
    
double &table3d::get(size_t ix, size_t iy, size_t z) {
  return (list[z])[ix][iy];
}

const double &table3d::get(size_t ix, size_t iy, size_t z) const {
  return (list[z])[ix][iy];
}

double &table3d::get_val_ret(double &x, double &y, size_t z) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];
  return (list[z])[ix][iy];
}

const double &table3d::get_val_ret(double &x, double &y, size_t z) const {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  x=xval[ix];
  y=yval[iy];
  return (list[z])[ix][iy];
}

double &table3d::get_val(double x, double y, size_t z) {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  return (list[z])[ix][iy];
}

const double &table3d::get_val(double x, double y, size_t z) const {
  size_t ix=0, iy=0;
  lookup_x(x,ix);
  lookup_y(y,iy);
  return (list[z])[ix][iy];
}

int table3d::set_grid_x(size_t ix, double val) {
  if (ix<numx) {
    (xval)[ix]=val;
    return o2scl::gsl_success;
  }
  O2SCL_ERR("Index out of range in set_grid_x().",o2scl::gsl_einval);
  return o2scl::gsl_einval;
}
    
int table3d::set_grid_y(size_t iy, double val) {
  if (iy<numy) {
    (yval)[iy]=val;
    return o2scl::gsl_success;
  }
  O2SCL_ERR("Index out of range in set_grid_y().",o2scl::gsl_einval);
  return o2scl::gsl_einval;
}
    
double table3d::get_grid_x(size_t ix) {
  if (ix<numx) {
    return (xval)[ix];
  }
  O2SCL_ERR("Index out of range in get_grid_x().",o2scl::gsl_einval);
  return 0.0;
}
    
double table3d::get_grid_y(size_t iy) {
  if (iy<numy) {
    return (yval)[iy];
  }
  O2SCL_ERR("Index out of range in get_grid_y().",o2scl::gsl_einval);
  return 0.0;
}

int table3d::get_size(size_t &nx, size_t &ny) const {
  nx=numx;
  ny=numy;
  return 0;
}

int table3d::get_nslices() const {
  return tree.size();
}

int table3d::line_of_names(std::string names) {
  int ret=0;
  std::string head;
      
  std::istringstream *is=new std::istringstream(names);
      
  while((*is) >> head) {
    o2scl::error_update(ret,new_slice(head));
  }
  delete is;
      
  if (ret!=0) {
    O2SCL_ERR_RET("At least one new column failed in line_of_names().",
		o2scl::gsl_efailed);
  }
      
  return o2scl::gsl_success;
}

std::string table3d::get_slice_name(size_t col) const {
  for(map_const_iter mit=tree.begin();mit!=tree.end();mit++) {
    if (mit->second==col) return mit->first;
  }
  O2SCL_ERR("Not found in get_slice_name().",o2scl::gsl_einval);
  return "";
}

int table3d::new_slice(std::string name) {
  int z;
  bool found=is_slice(name,z);
  if (found==true) {
    O2SCL_ERR_RET("Already a slice with the same name in new_slice().",
		o2scl::gsl_einval);
  }
  o2scl::omatrix mp(numx,numy);
  list.push_back(mp);
  tree.insert(make_pair(name,list.size()-1));
  has_slice=true;
  return 0;
}

int table3d::lookup_slice(std::string name, size_t &ix) const {
  for(map_const_iter mit=const_begin();mit!=const_end();mit++) {
    if (mit->first==name) {
      ix=mit->second;
      return o2scl::gsl_success;
    }
  }
  O2SCL_ERR("Not found in lookup_slice().",o2scl::gsl_einval);
  return o2scl::gsl_einval;
}

bool table3d::is_slice(std::string name, int &ix) {
  for(map_iter mit=begin();mit!=end();mit++) {
    if (mit->first==name) {
      ix=mit->second;
      return true;
    }
  }
  return false;
}

int table3d::rename_slice(std::string olds, std::string news) {
  size_t oi, ni;
  int ret=lookup_slice(olds,oi);
  if (ret!=0) {
    O2SCL_ERR_RET("Failed to find in rename_slice().",gsl_efailed);
  }

  ret=new_slice(news);
  if (ret!=0) {
    O2SCL_ERR_RET("Failed to create new slice in rename_slice().",
		gsl_efailed);
  }
  ret=lookup_slice(news,ni);
  if (ret!=0) {
    O2SCL_ERR_RET("Failed to find new slice in rename_slice().",
		      gsl_efailed);
  }
  
  for(size_t i=0;i<numx;i++) {
    for(size_t j=0;j<numy;j++) {
      list[ni][i][j]=list[oi][i][j];
    }
  }
  return gsl_success;
}

int table3d::copy_slice(std::string src, std::string dest) {
  size_t sl1, sl2;
  int ret=lookup_slice(src,sl1);
  if (ret==0) {
    ret=new_slice(dest);
  } else {
    O2SCL_ERR_RET("Failed to find source slice in copy_size().",
		o2scl::gsl_einval);
  }
  if (ret==0) {
    ret=lookup_slice(dest,sl2);
  } else {
    O2SCL_ERR("Failed to create new slice in copy_slice().",
	    o2scl::gsl_einval);
    return ret;
  }
  if (ret==0) {
    for(size_t i=0;i<numx;i++) {
      for(size_t j=0;j<numy;j++) {
	(list[sl2])[i][j]=(list[sl1])[i][j];
      }
    }
  } else {
    O2SCL_ERR_RET("Failed to find destination slice in copy_slice()!",
		o2scl::gsl_esanity);
  }
  return o2scl::gsl_success;
}
  
int table3d::init_slice(std::string scol, double val) {
  if (!finite(val)) {
    O2SCL_ERR_RET("Value not finite in init_slice()",o2scl::gsl_einval);
  }
  size_t sl1;
  int ret=lookup_slice(scol,sl1);
  if (ret==0) {
    for(size_t i=0;i<numx;i++) {
      for(size_t j=0;j<numy;j++) {
	(list[sl1])[i][j]=val;
      }
    }
  } else {
    O2SCL_ERR_RET((((std::string)"Slice '")+scol+
		 "' not found in init_slice()").c_str(),
		o2scl::gsl_enotfound);
  }
  return o2scl::gsl_success;
}
  
int table3d::lookup_x(double val, size_t &ix) const {
  if (numx>0) {
    double dist=fabs(val-(xval)[0]);
    ix=0;
    for(size_t i=1;i<numx;i++) {
      if (fabs(val-(xval)[i])<dist) {
	ix=i;
	dist=fabs(val-(xval)[i]);
      }
    }
    return 0;
  }
  return o2scl::gsl_einval;
}
  
int table3d::lookup_y(double val, size_t &iy) const {
  if (numy>0) {
    double dist=fabs(val-(yval)[0]);
    iy=0;
    for(size_t i=1;i<numy;i++) {
      if (fabs(val-(yval)[i])<dist) {
	iy=i;
	dist=fabs(val-(yval)[i]);
      }
    }
    return 0;
  }
  return o2scl::gsl_einval;
}

int table3d::lookup(double val, std::string slice, size_t &ix, 
		    size_t &iy) const {
  if (numx>0 && numy>0) {
    double dist=fabs(get(0,0,slice)-val);
    ix=0;
    iy=0;
    for(size_t i=0;i<numx;i++) {
      for(size_t j=0;j<numy;j++) {
	if (fabs(get(i,j,slice)-val)<dist) {
	  dist=fabs(get(i,j,slice)-val);
	  ix=i;
	  iy=j;
	}
      }
    }
    return 0;
  }
  return o2scl::gsl_einval;
}

int table3d::set_interp(o2scl::base_interp_mgr<o2scl::ovector_const_view> &b1,
			o2scl::base_interp_mgr
			<o2scl::ovector_const_subvector> &b2) {
  bimp1=&b1;
  bimp2=&b2;
  return 0;
}

double table3d::interp(double x, double y, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }
  
  sm_interp_vec **sii=new sm_interp_vec *[numy];
  omatrix_col **acii=new omatrix_col *[numy];
  for(size_t i=0;i<numy;i++) {
    acii[i]=new omatrix_col(list[z],i);
    sii[i]=new sm_interp_vec(*bimp1,*bimp2,numx,xval,*(acii[i]));
  }
      
  ovector icol(numy);
  for(size_t i=0;i<numy;i++) {
    icol[i]=sii[i]->interp(x);
  }
  sm_interp_vec *siy=
    new sm_interp_vec(*bimp1,*bimp2,numy,yval,icol);
  result=siy->interp(y);
      
  for(size_t i=0;i<numy;i++) {
    delete sii[i];
    delete acii[i];
  }
  delete[] sii;
  delete[] acii;
  delete siy;

  return result;
}

double table3d::deriv_x(double x, double y, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }
      
  si=new o2scl::sm_interp_vec *[numy];
  aci=new o2scl::omatrix_col *[numy];
  for(size_t i=0;i<numy;i++) {
    aci[i]=new o2scl::omatrix_col(list[z],i);
    si[i]=new o2scl::sm_interp_vec(*bimp1,*bimp2,numx,xval,*(aci[i]));
  }
      
  o2scl::ovector icol(numy);
  for(size_t i=0;i<numy;i++) {
    icol[i]=si[i]->deriv(x);
  }
  o2scl::sm_interp_vec *siy=
    new o2scl::sm_interp_vec(*bimp1,*bimp2,numy,yval,icol);
  result=siy->interp(y);
      
  for(size_t i=0;i<numy;i++) {
    delete si[i];
    delete aci[i];
  }
  delete[] si;
  delete[] aci;
  delete siy;

  return result;
}

double table3d::integ_y(double x, double y1, double y2, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }
      
  si=new o2scl::sm_interp_vec *[numy];
  aci=new o2scl::omatrix_col *[numy];
  for(size_t i=0;i<numy;i++) {
    aci[i]=new o2scl::omatrix_col(list[z],i);
    si[i]=new o2scl::sm_interp_vec(*bimp1,*bimp2,numx,xval,*(aci[i]));
  }
      
  o2scl::ovector icol(numy);
  for(size_t i=0;i<numy;i++) {
    icol[i]=si[i]->deriv(x);
  }
  o2scl::sm_interp_vec *siy=
    new o2scl::sm_interp_vec(*bimp1,*bimp2,numy,yval,icol);
  result=siy->integ(y1,y2);
      
  for(size_t i=0;i<numy;i++) {
    delete si[i];
    delete aci[i];
  }
  delete[] si;
  delete[] aci;
  delete siy;

  return result;
}

double table3d::integ_x(double x1, double x2, double y, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }

  o2scl::omatrix_row **ari;
  si=new o2scl::sm_interp_vec *[numx];
  ari=new o2scl::omatrix_row *[numx];
  for(size_t i=0;i<numx;i++) {
    ari[i]=new o2scl::omatrix_row(list[z],i);
    si[i]=new o2scl::sm_interp_vec(*bimp1,*bimp2,numy,yval,*(ari[i]));
  }
      
  o2scl::ovector icol(numx);
  for(size_t i=0;i<numx;i++) {
    icol[i]=si[i]->deriv(y);
  }
  o2scl::sm_interp_vec *six=
    new o2scl::sm_interp_vec(*bimp1,*bimp2,numx,xval,icol);
  result=six->integ(x1,x2);
      
  for(size_t i=0;i<numx;i++) {
    delete si[i];
    delete ari[i];
  }
  delete[] si;
  delete[] ari;
  delete six;

  return result;
}

double table3d::deriv_y(double x, double y, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }
      
  si=new o2scl::sm_interp_vec *[numy];
  aci=new o2scl::omatrix_col *[numy];
  for(size_t i=0;i<numy;i++) {
    aci[i]=new o2scl::omatrix_col(list[z],i);
    si[i]=new o2scl::sm_interp_vec(*bimp1,*bimp2,numx,xval,*(aci[i]));
  }
      
  o2scl::ovector icol(numy);
  for(size_t i=0;i<numy;i++) {
    icol[i]=si[i]->interp(x);
  }
  o2scl::sm_interp_vec *siy=
    new o2scl::sm_interp_vec(*bimp1,*bimp2,numy,yval,icol);
  result=siy->deriv(y);
      
  for(size_t i=0;i<numy;i++) {
    delete si[i];
    delete aci[i];
  }
  delete[] si;
  delete[] aci;
  delete siy;

  return result;
}

double table3d::deriv_xy(double x, double y, std::string name) {
  double result;
      
  size_t z=0; int ret=lookup_slice(name,z);
  if (ret!=0) {
    O2SCL_ERR("Failed to find slice in interp().",o2scl::gsl_einval);
    return 0.0;
  }
      
  si=new o2scl::sm_interp_vec *[numy];
  aci=new o2scl::omatrix_col *[numy];
  for(size_t i=0;i<numy;i++) {
    aci[i]=new o2scl::omatrix_col(list[z],i);
    si[i]=new o2scl::sm_interp_vec(*bimp1,*bimp2,numx,xval,*(aci[i]));
  }
      
  o2scl::ovector icol(numy);
  for(size_t i=0;i<numy;i++) {
    icol[i]=si[i]->deriv(x);
  }
  o2scl::sm_interp_vec *siy=
    new o2scl::sm_interp_vec(*bimp1,*bimp2,numy,yval,icol);
  result=siy->deriv(y);
      
  for(size_t i=0;i<numy;i++) {
    delete si[i];
    delete aci[i];
  }
  delete[] si;
  delete[] aci;
  delete siy;

  return result;
}

int table3d::zero_table() {
  for(size_t i=0;i<list.size();i++) {
    for(size_t j=0;j<numx;j++) {
      for(size_t k=0;k<numy;k++) {
	list[i][j][k]=0.0;
      }
    }
  }
  return 0;
}

void table3d::clear_table() {
  clear_data();
  xval.free();
  yval.free();
  numx=0;
  numy=0;
  has_slice=false;
  xy_set=false;
  size_set=false;
  return;
}

void table3d::clear_data() {
  tree.clear();
  for(size_t i=0;i<list.size();i++) {
    list[i].free();
  }
  list.clear();
      
  has_slice=false;
  return;
}

int table3d::summary(std::ostream *out, int ncol) const {
      
  if (constants.size()==1) {
    (*out) << "1 constant:" << std::endl;
  } else {
    (*out) << constants.size() << " constants:" << std::endl;
  }
  std::map<std::string,double>::const_iterator mit;
  for(mit=constants.begin();mit!=constants.end();mit++) {
    (*out) << mit->first << " " << mit->second << std::endl;
  }
      
  // Output number of slices
  int nh=get_nslices();
  if (nh==1) {
    (*out) << "1 slice: " << std::endl;
  } else {
    (*out) << nh << " slices: " << std::endl;
  }
  std::string *h=new std::string[nh];
  for(int i=0;i<nh;i++) {
    h[i]=o2scl::itos(i)+". "+get_slice_name(i);
  }
      
  // Convert to string with width 'ncol'
  std::vector<std::string> h2;
  o2scl::screenify(nh,h,h2);
  size_t nh2=h2.size();
      
  // Output slice names
  for(size_t i=0;i<nh2;i++) {
    (*out) << h2[i] << std::endl;
  }
      
  if (numx>0) {
    (*out) << "X-grid " << xname << ", size: " 
	   << numx << " First: " << (xval)[0] 
	   << " Last: " << (xval)[numx-1] << std::endl;
  } else {
    (*out) << "No x-grid." << std::endl;
  }

  if (numy>0) {
    (*out) << "Y-grid " << yname << ", size: " 
	   << numy << " First: " << (yval)[0] 
	   << " Last: " << (yval)[numy-1] << std::endl;
  } else {
    (*out) << "No y-grid." << std::endl;
  }

  delete[] h;
      
  return 0;
}

int table3d::add_constant(std::string name, double val) {
  if (constants.find(name)!=constants.end()) {
    constants.find(name)->second=val;
    return 0;
  }
  constants.insert(make_pair(name,val));
  return 0;
  return 0;
}

int table3d::remove_constant(std::string name) {
  constants.erase(name);
  return 0;
}

int table3d::set_constant(std::string name, double val) {
  if (constants.find(name)!=constants.end()) {
    constants.find(name)->second=val;
    return 0;
  }
  O2SCL_ERR_RET("No constant with specified name in set_constant().",
	      gsl_einval);
}

double table3d::get_constant(std::string name) {
  if (constants.find(name)!=constants.end()) {
    return constants.find(name)->second;
  }
  O2SCL_ERR("No constant with specified name in get_constant().",
	  gsl_einval);
  return 0.0;
}

int table3d::extract_x(double x, table &t) {
  t.clear_table();
  
  string s=yname+" ";;
  for(size_t i=0;i<tree.size();i++) {
    s+get_slice_name(i)+" ";
  }
  t.line_of_names(s);
  t.set_nlines(numy);

  for(size_t i=0;i<numy;i++) {
    t.set(yname,i,yval[i]);
    for(size_t j=0;j<tree.size();j++) {
      string tmp=get_slice_name(j);
      t.set(tmp,i,interp(x,yval[i],tmp));
    }
  }
  
  return 0;
}

int table3d::extract_y(double y, table &t) {
  t.clear_table();
  
  string s=xname+" ";;
  for(size_t i=0;i<tree.size();i++) {
    s+get_slice_name(i)+" ";
  }
  t.line_of_names(s);
  t.set_nlines(numx);

  for(size_t i=0;i<numx;i++) {
    t.set(xname,i,xval[i]);
    for(size_t j=0;j<tree.size();j++) {
      string tmp=get_slice_name(j);
      t.set(tmp,i,interp(y,xval[i],tmp));
    }
  }
  
  return 0;
}
   
const o2scl::omatrix &table3d::get_slice(std::string name) const {
  size_t z=0;
  int ret=lookup_slice(name,z);
  return list[z];
}

const o2scl::omatrix &table3d::get_slice(size_t iz) const {
  return list[iz];
}

const std::vector<o2scl::omatrix> &table3d::get_data() const {
  return list;
}
