/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_TABLE_UNITS_H
#define O2SCL_TABLE_UNITS_H

#include <o2scl/table.h>
#include <o2scl/convert_units.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Data \table class with units

      \todo Make table methods virtual? (not necessary yet
      since delete_column() isn't referred to internally)

      acol commands which need editing for table_units
      - binary 
      - html
  */
  class table_units : public table {
  public:
  
    /** \brief Create a new table_units with space for nlines<=cmaxlines.
     */
    table_units(int cmaxlines=0);

    virtual ~table_units();

    /// \name Copy constructors
    //@{
    table_units(const table_units &t);
    table_units(const table &t);
    table_units &operator=(const table_units &t);
    table_units &operator=(const table &t);
    //@}

    /// Get the unit for column \c scol
    std::string get_unit(std::string scol) const;

    /// Remove the unit for column \c scol
    int remove_unit(std::string scol);
    
    /// Set the unit for column \c scol to \c unit
    int set_unit(std::string scol, std::string unit);

    /// Convert the units of column \c scol to \c unit
    int convert_to_unit(std::string scol, std::string unit,
			bool err_on_fail=true);

    /// Get the conversion factor from \c old_unit to \c new_unit
    double get_conv(std::string old_unit, std::string new_unit);

    /// The default object for unit conversions
    convert_units def_cu;

    /// Delete column named \c scol
    virtual int delete_column(std::string scol);

    /// Return the type, \c "table_units".
    virtual const char *type() { return "table_units"; }

    /// Output a summary of the information stored
    virtual int summary(std::ostream *out, int ncol=79) const;

    /// Set the convert units object
    int set_convert(convert_units &c) {
      cup=&c;
      return 0;
    }

    /// Show the units
    int show_units() {
      cup->print_cache();
      return 0;
    }

  protected:

#ifndef DOXYGEN_INTERNAL
    
    friend class io_tlate<table_units>;

    /// The pointer to the convert units object
    convert_units *cup;

    /// \name Unit map iterator types
    //@{
    typedef std::map<std::string,std::string,
      string_comp>::iterator uiter;
    typedef std::map<std::string,std::string,
      string_comp>::const_iterator uciter;
    //@}
  
    /// Unit map
    std::map<std::string,std::string,string_comp> utree;
  
#endif

  };

  /// Input specialization
  template<> int io_tlate<table_units>::input
    (cinput *co, o2scl::in_file_format *ins, table_units *ta);
  /// Output specialization
  template<> int io_tlate<table_units>::output
    (coutput *co, o2scl::out_file_format *outs, 
     table_units *at);
  /// \fn Type specialization
  template<> const char *io_tlate<table_units>::type();
  
  typedef io_tlate<table_units> table_units_io_type;
  
#ifndef DOXYGENP
}
#endif

#endif
