/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/test_mgr.h>
#include <o2scl/gsl_deriv.h>

using namespace std;
using namespace o2scl;

double testfun(double x, int &pa) {
  return sin(x);
}

int main(void) {
  int vp=0;
  double res;
  test_mgr t;
  t.set_output_level(2);
  def_err_hnd.set_mode(2);

  cout.setf(ios::scientific);
  cout.precision(10);

  gsl_deriv<int ,funct<int > > de;

  funct_fptr_noerr<int > tf(testfun);

  de.h=1.0e-4;

  res=de.calc(0.5,vp,tf);
  cout << "First derivative: " << endl;
  cout << res << " " << de.get_err() 
       << " " << cos(0.5) << endl;
  t.test_rel(res,cos(0.5),1.0e-11,"simple derivative");

  cout << "Second derivative: " << endl;
  res=de.calc2(0.5,vp,tf);
  cout << res << " " 
       << de.get_err() << " " << -sin(0.5) << endl;
  t.test_rel(res,-sin(0.5),1.0e-6,"second derivative");

  cout << "Third derivative: " << endl;
  res=de.calc3(0.5,vp,tf);
  cout << res << " " 
  << de.get_err() << " " << -cos(0.5) << endl;
  t.test_rel(res,-cos(0.5),1.0e-2,"third derivative");
  cout << endl;

  de.verbose=1;
  res=de.calc(0.5,vp,tf);

  t.report();
  return 0;
}
