/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/bag_eos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

bag_eos::bag_eos() {
  bag_constant=200.0/hc_mev_fm;
}

int bag_eos::calc_p(quark &u, quark &d, quark &s, thermo &th) {

  if (u.mu>u.m) u.kf=sqrt(u.mu*u.mu-u.m*u.m);
  else u.kf=0.0;
  if (d.mu>d.m) d.kf=sqrt(d.mu*d.mu-d.m*d.m);
  else d.kf=0.0;
  if (s.mu>s.m) s.kf=sqrt(s.mu*s.mu-s.m*s.m);
  else s.kf=0.0;

  u.ms=u.m;
  d.ms=d.m;
  s.ms=s.m;
  
  u.calc_mu_zerot();
  d.calc_mu_zerot();
  s.calc_mu_zerot();

  th.ed=u.ed+d.ed+s.ed+bag_constant;
  th.pr=u.pr+d.pr+s.pr-bag_constant;
  th.en=0.0;
  
  return 0;
}

int bag_eos::calc_e(quark &u, quark &d, quark &s, thermo &th) {

  u.kf_from_density();
  d.kf_from_density();
  s.kf_from_density();

  u.mu=sqrt(u.kf*u.kf+u.m*u.m);
  d.mu=sqrt(d.kf*d.kf+d.m*d.m);
  s.mu=sqrt(s.kf*s.kf+s.m*s.m);

  u.ms=u.m;
  d.ms=d.m;
  s.ms=s.m;

  u.calc_density_zerot();
  d.calc_density_zerot();
  s.calc_density_zerot();

  th.ed=u.ed+d.ed+s.ed+bag_constant;
  th.pr=u.pr+d.pr+s.pr-bag_constant;
  th.en=0.0;
  
  return 0;
}

int bag_eos::calc_temp_p(quark& u, quark& d, quark& s, 
			      const double temper, thermo &th) {
  int ret=0;

  if (temper<=0.0) {
    return calc_p(u,d,s,th);
  }

  u.nu=u.mu;
  d.nu=d.mu;
  s.nu=s.mu;
  u.ms=u.m;
  d.ms=d.m;
  s.ms=s.m;
  
  ret=u.pair_mu(temper);
  if (ret!=0) {
    O2SCL_ERR("Up quark failed in bag_eos::calc_temp_p(). Continuing.",
	    gsl_efailed);
  }
  ret=d.pair_mu(temper);
  if (ret!=0) {
    O2SCL_ERR("Down quark failed in bag_eos::calc_temp_p(). Continuing.",
	    gsl_efailed);
  }
  ret=s.pair_mu(temper);
  if (ret!=0) {
    O2SCL_ERR("Strange quark failed in bag_eos::calc_temp_p(). Continuing.",
	    gsl_efailed);
  }
  
  u.kf_from_density();
  d.kf_from_density();
  s.kf_from_density();
  
  th.ed=u.ed+d.ed+s.ed+bag_constant;
  th.pr=u.pr+d.pr+s.pr-bag_constant;
  th.en=(th.ed+th.pr-u.n*u.mu-d.n*d.mu-s.n*s.mu)/
    temper;

  return ret;

}

int bag_eos::calc_temp_e(quark& u, quark& d, quark& s, 
			      const double temper, thermo &th) {
  int ret=0;

  u.ms=u.m;
  d.ms=d.m;
  s.ms=s.m;
  
  ret=u.pair_density(temper);
  ret=ret || (d.pair_density(temper));
  ret=ret || (s.pair_density(temper));
  
  u.kf_from_density();
  d.kf_from_density();
  s.kf_from_density();
  
  th.ed=u.ed+d.ed+s.ed+bag_constant;
  th.pr=u.pr+d.pr+s.pr-bag_constant;
  th.en=(th.ed+th.pr-u.n*u.mu-d.n*d.mu-s.n*s.mu)/
    temper;

  return ret;

}

