/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/nse_eos.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

nse_eos::nse_eos() {
  root=&def_root;
}

int nse_eos::calc_mu(double mun, double mup, double T,
		     double &nb, double &Ye, thermo &th, nuclear_dist &nd) {

  nb=0.0;
  Ye=0.0;
  
  int ic=0;
  for (nuclear_dist::iterator ndi=nd.begin();ndi!=nd.end();ndi++) {
    ndi->mu=mun*ndi->N+mup*ndi->Z-ndi->be;
    ndi->non_interacting=true;
    ndi->calc_mu(T);
    nb+=ndi->n*((double)ndi->A);
    Ye+=ndi->n*((double)ndi->Z);
    th.ed+=ndi->ed;
    th.pr+=ndi->pr;
    th.en+=ndi->en;
  }

  Ye/=nb;
  
  return 0;
}

int nse_eos::calc_density(double nb, double Ye, double T, 
			  double &mun, double &mup, thermo &th, 
			  nuclear_dist &nd) {

  ovector x(2);
  x[0]=mun;
  x[1]=mup;
  mm_funct_mfptr<nse_eos,solve_parms> mfm(this,&nse_eos::solve_fun);

  solve_parms sp={nb,Ye,T,&nd};

  root->msolve(2,x,sp,mfm);

  mun=x[0];
  mup=x[1];
  
  calc_mu(mun,mup,T,nb,Ye,th,nd);

  return gsl_success;
}

int nse_eos::solve_fun(size_t nv, const ovector_base &x, ovector_base &y, 
		       solve_parms &sp) {
  double mun=x[0];
  double mup=x[1];

  double nb, Ye;
  thermo th;
  
  calc_mu(mun,mup,sp.T,nb,Ye,th,*sp.ndp);
  
  y[0]=(nb-sp.nb)/sp.nb;
  y[1]=Ye-sp.Ye;

  return gsl_success;
}

