/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/nse_eos.h>
#include <o2scl/nuclear_mass.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

int main(void) {

  test_mgr t;
  t.set_output_level(2);

  double nb, Ye, mun=0.0, mup=0.0, nbnew;
  double T=5.0/hc_mev_fm;
  thermo th;

  cout.setf(ios::scientific);

  mnmsk_mass mm;
  simple_dist sd(24,32,55,58,mm);
  
  nse_eos ne;

  // Initial guess
  mun=-6.89e-2;
  mup=-2.11e-2;

  int ret=ne.calc_density(0.0855,0.49,2.0/hc_mev_fm,mun,mup,th,sd);
  cout << ret << " " << mun << " " << mup << endl;
  
  nbnew=0.0;
  double nbnew2=0.0;
  for(nuclear_dist::iterator ndi=sd.begin();ndi!=sd.end();ndi++) {
    if (ndi->n*ndi->A>1.0e-5) {
      cout << ndi->Z << " " << mm.Ztoel(ndi->Z) << " " 
	   << ndi->A << " " << ndi->n << endl;
      nbnew2+=ndi->n*ndi->A;
    }
    nbnew+=ndi->n*ndi->A;
  }
  cout << nbnew << " " << 0.0855 << " " << nbnew2 << endl;
  t.test_rel(nbnew,0.0855,1.0e-3,"nb match.");
  
  t.report();
  return 0;
}


