/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/funct.h>
#include <o2scl/cern_adapt.h>

using namespace std;
using namespace o2scl;

double testfun(double tx, double &pa);

funct_fptr_noerr<double> tf(testfun);

double testfun(double tx, double &a) {
  return -cos(1.0/(tx+a))/(a+tx)/(a+tx);
}

int main(void) {
  double a, calc, exact, diff, ei;
  cern_adapt<double,funct_fptr_noerr<double> > cg;
  test_mgr t;
  t.set_output_level(2);
  def_err_hnd.set_mode(2);

  cout.setf(ios::scientific);
  cout.precision(10);
  
  a=0.01;
  cg.integ_err(tf,0.0,1.0,a,calc,ei);
  exact=sin(1.0/(1.0+a))-sin(1.0/a);
  t.test_rel(calc,exact,1.0e-8,"cern_adapt");
  diff=fabs(calc-exact);
  cout << calc << " " << exact << " " << diff << " " << ei << endl;

  // This is a nasty function and takes many segments (68)
  cout << cg.get_nsegments() << endl;
  size_t n=cg.get_nsegments();
  ovector xlo(n), xhi(n), val(n), err(n);
  cg.get_segments(xlo,xhi,val,err);
  for(size_t i=0;i<n;i+=10) {
    cout << xlo[i] << " " << xhi[i] << " ";
    cout.setf(ios::showpos);
    cout << val[i] << " ";
    cout.unsetf(ios::showpos);
    cout << err[i] << endl;
  }

  a=0.01;
  cg.verbose=1;
  cout.precision(6);
  cg.integ_err(tf,0.0,1.0,a,calc,ei);
  cout.precision(10);
  exact=sin(1.0/(1.0+a))-sin(1.0/a);
  t.test_rel(calc,exact,1.0e-8,"cern_adapt");
  diff=fabs(calc-exact);
  cout << calc << " " << exact << " " << diff << " " << ei << endl;

  t.report();
  return 0;
}

