/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qng.h>

using namespace std;
using namespace o2scl;

double testfun1(double x, int &pa) {
  return x*x;
}

double testfun2(double x, int &pa) {
  return -sin(1.0/(x+0.01))*pow(x+0.01,-2.0);
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  def_err_hnd.set_mode(2);
  gsl_inte_qng<int, funct_fptr_noerr<int> > it1;

  double ans, exact;
  int vp=0;

  cout.setf(ios::scientific);
  cout.precision(10);

  funct_fptr_noerr<int> tf1(testfun1);
  funct_fptr_noerr<int> tf2(testfun2);

  cout << "A simple integral for QNG: " << endl;
  ans=it1.integ(tf1,0.0,1.0,vp);
  exact=1.0/3.0;
  cout << "Results: ";
  cout << ans << " " << it1.get_error() << " " << exact << " "
       << fabs(ans-exact) << endl;
  t.test_rel(ans,exact,1.0e-14,"qng test");
  cout << endl;

  cout << "A complex integral: gsl_inte_qng fails:" << endl;
  def_err_hnd.set_mode(1);
  ans=it1.integ(tf2,0.0,1.0,vp);
  def_err_hnd.set_mode(2);
  cout << endl;

  cout << "Results: ";
  exact=cos(100.0)-cos(1/1.01);
  cout << ans << " " << it1.get_error() << " " << exact << " "
       << fabs(ans-exact) << endl;
  t.test_rel(ans,exact,1.0,"qng failure");
  cout << endl;

  t.report();
  return 0;
}
