/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>
#include <o2int/graph.h>
#include <o2scl/test_mgr.h>
#include <o2scl/contour.h>

using namespace std;
using namespace o2scl;

double fun(double x, double y) {
  return 15.0*exp(-pow(x-20.0,2.0)/400.0-pow(y-5.0,2.0)/25.0)
    +40.0*exp(-pow(x-70.0,2.0)/4900.0-pow(y-2.0,2.0)/4.0);
}

int main(void) {
  int i, j;
  test_mgr t;
  t.set_output_level(2);

  ifstream fin("../other/contour.dat");
  int nc, *sz;
  double **xcont, **ycont;

  TApplication theApp("App",0,NULL);
  TCanvas *c1, *c2, *c3, *c4;
  TPad *p1, *p2, *p3, *p4;
  TH1 *th1, *th2, *th3, *th4;

  // ---------------------------------------------

  fin >> nc;
  sz=new int[nc];
  xcont=new double *[nc];
  ycont=new double *[nc];
  for(i=0;i<nc;i++) {
    fin >> sz[i];
    xcont[i]=new double[sz[i]];
    ycont[i]=new double[sz[i]];
    for(j=0;j<sz[i];j++) {
      double dtmp1, dtmp2;
      fin >> xcont[i][j] >> ycont[i][j];
    }
  }
  
  o2scl_graph::new_graph(c1,p1,th1,"c1","cc1","p1",0.0,0.0,81.0,9.0,
			 0,0,700,700);
  
  for(i=0;i<nc;i++) {
    TGraph *g1=new TGraph(sz[i],xcont[i],ycont[i]);
    g1->Draw();
  }

  // ---------------------------------------------

  fin >> nc;
  sz=new int[nc];
  xcont=new double *[nc];
  ycont=new double *[nc];
  for(i=0;i<nc;i++) {
    fin >> sz[i];
    xcont[i]=new double[sz[i]];
    ycont[i]=new double[sz[i]];
    for(j=0;j<sz[i];j++) {
      double dtmp1, dtmp2;
      fin >> xcont[i][j] >> ycont[i][j];
    }
  }

  o2scl_graph::new_graph(c2,p2,th2,"c2","cc2","p2",0.0,0.0,8.0,3.0,
		       0,0,700,700);
  
  for(i=0;i<nc;i++) {
    TGraph *g1=new TGraph(sz[i],xcont[i],ycont[i]);
    g1->Draw();
  }

  // ---------------------------------------------

  fin >> nc;
  sz=new int[nc];
  xcont=new double *[nc];
  ycont=new double *[nc];
  for(i=0;i<nc;i++) {
    fin >> sz[i];
    xcont[i]=new double[sz[i]];
    ycont[i]=new double[sz[i]];
    for(j=0;j<sz[i];j++) {
      double dtmp1, dtmp2;
      fin >> xcont[i][j] >> ycont[i][j];
    }
  }
  
  o2scl_graph::new_graph(c3,p3,th3,"c3","cc3","p3",0.0,0.0,8.0,3.0,
		       0,0,700,700);
  
  for(i=0;i<nc;i++) {
    TGraph *g1=new TGraph(sz[i],xcont[i],ycont[i]);
    g1->Draw();
  }
  for (size_t i=0;i<8;i++) {
    TLine *l1=new TLine(((double)i)*8.0/7.0,0,((double)i)*8.0/7.0,3);
    l1->Draw();
  }
  for (size_t i=0;i<10;i++) {
    TLine *l2=new TLine(0,((double)i)*3.0/9.0,8,((double)i)*3.0/9.0);
    l2->Draw();
  }
  c3->Print("contour3.pdf");

  // ---------------------------------------------

  fin >> nc;
  sz=new int[nc];
  xcont=new double *[nc];
  ycont=new double *[nc];
  for(i=0;i<nc;i++) {
    fin >> sz[i];
    xcont[i]=new double[sz[i]];
    ycont[i]=new double[sz[i]];
    for(j=0;j<sz[i];j++) {
      double dtmp1, dtmp2;
      fin >> xcont[i][j] >> ycont[i][j];
    }
  }
  
  o2scl_graph::new_graph(c4,p4,th4,"c4","cc4","p4",0.0,0.0,6.1,6.1,
		       0,0,700,700);
  
  for(i=0;i<nc;i++) {
    TGraph *g1=new TGraph(sz[i],xcont[i],ycont[i]);
    g1->Draw();
  }
  c4->Print("contour4.pdf");

  // ---------------------------------------------

  fin.close();

  theApp.Run(kTRUE);
  
  t.report();
  return 0;
}
