/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <iostream>
#include <o2scl/constants.h>
#include <o2scl/test_mgr.h>
#include <o2scl/gsl_chebapp.h>

using namespace std;
using namespace o2scl;

double func(double x, int &vp) {
  return sin(x);
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);

  gsl_chebapp gc;
  int vp=0;

  // Initialize a Chebyshev approximation of the function y=sin(x)
  gc.init(func,10,0.0,2.0*o2scl_const::pi,vp);
  t.test_rel(gc.eval(0.55),sin(0.55),5.0e-2,"cheb1");
  cout << gc.eval(0.55) << " " << sin(0.55) << endl;
  cout << gc.eval_n(5,0.55) << " " << sin(0.55) << endl;
  double res, err;
  gc.eval_err(0.55,res,err);
  cout << res << " " << err << endl;
  gc.eval_n_err(5,0.55,res,err);
  cout << res << " " << err << endl;

  // Try operator=()
  gsl_chebapp gc2=gc;
  t.test_rel(gc2.eval(0.55),sin(0.55),5.0e-2,"cheb1");
  cout << gc2.eval(0.55) << " " << sin(0.55) << endl;

  // Show how to get the derivative
  gsl_chebapp gc3;
  gc.deriv(gc3);
  t.test_rel(gc3.eval(0.55),cos(0.55),5.0e-2,"dcheb2");
  cout << gc3.eval(0.55) << " " << cos(0.55) << endl;

  // Show how to get the integral
  gsl_chebapp gc4;
  gc.integ(gc4);
  t.test_rel(gc4.eval(0.55)-gc4.eval(0.45),-cos(0.55)+cos(0.45),
	     5.0e-2,"icheb2");
  cout << gc4.eval(0.55) << " " << -cos(0.55)+cos(0.45) << endl;

  // Show how to use the eval() function to represent
  // the Chebyshev approximation as a funct object
  funct_cmfptr_nopar<gsl_chebapp,int> fmn(&gc,&gsl_chebapp::eval);
  double y;
  for(double x=0.0;x<1.01;x+=0.2) {
    fmn(x,y,vp);
    cout << y << " " << sin(x) << endl;
    t.test_rel(y,sin(x),5.0e-2,"as func");
  }

  t.report();
  return 0;
}


